@@ -3,19 +3,19 @@
  * Copyright (c) 2008 Jasper Huijsmans <jasper@xfce.org>.
  * Copyright (c) 2008 Jannis Pohlmann <jannis@xfce.org>.
  *
- * This program is free software; you can redistribute it and/or modify 
- * it under the terms of the GNU General Public License as published by 
- * the Free Software Foundation; either version 2 of the License, or (at 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but 
- * WITHOUT ANY WARRANTY; without even the implied warranty of 
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License 
- * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  * MA  02111-1307  USA
  */
 
@@ -32,9 +32,9 @@
 #include <gtk/gtk.h>
 
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
-#include <libxfce4menu/libxfce4menu.h>
-#include <thunar-vfs/thunar-vfs.h>
+#include <libxfce4ui/libxfce4ui.h>
+#include <garcon/garcon.h>
+#include <gio/gio.h>
 #include <xfconf/xfconf.h>
 
 #include "xfce-appfinder-window.h"
@@ -53,6 +53,7 @@
 #define TEXT_COLUMN 1
 #define CATEGORY_COLUMN 2
 #define ITEM_COLUMN 3
+#define TOOLTIP_COLUMN 4
 
 
 
@@ -65,8 +66,6 @@ enum
 
 
 
-static void       xfce_appfinder_window_class_init         (XfceAppfinderWindowClass *klass);
-static void       xfce_appfinder_window_init               (XfceAppfinderWindow      *window);
 static void       xfce_appfinder_window_constructed        (GObject                  *object);
 static void       xfce_appfinder_window_dispose            (GObject                  *object);
 static void       xfce_appfinder_window_finalize           (GObject                  *object);
@@ -84,7 +83,7 @@ static void       _xfce_appfinder_window_entry_changed     (GtkEditable
                                                             XfceAppfinderWindow      *window);
 static void       _xfce_appfinder_window_entry_activated   (GtkEntry                 *entry,
                                                             XfceAppfinderWindow      *window);
-static void       _xfce_appfinder_window_entry_focused     (GtkWidget                *entry,
+static gboolean   _xfce_appfinder_window_entry_focused     (GtkWidget                *entry,
                                                             GdkEventFocus            *event,
                                                             XfceAppfinderWindow      *window);
 static gboolean   _xfce_appfinder_window_entry_key_pressed (GtkWidget                *widget,
@@ -96,7 +95,7 @@ static gboolean   _xfce_appfinder_window_radio_key_pressed (GtkWidget
 static gboolean   _xfce_appfinder_window_view_key_pressed  (GtkWidget                *widget,
                                                             GdkEventKey              *event,
                                                             XfceAppfinderWindow      *window);
-static void       _xfce_appfinder_window_category_changed  (XfceAppfinderWindow      *window, 
+static void       _xfce_appfinder_window_category_changed  (XfceAppfinderWindow      *window,
                                                             GtkToggleButton          *button);
 static void       _xfce_appfinder_window_cursor_changed    (GtkTreeView              *tree_view,
                                                             XfceAppfinderWindow      *window);
@@ -104,25 +103,14 @@ static void       _xfce_appfinder_window_drag_data_get     (GtkWidget
                                                             GdkDragContext           *drag_context,
                                                             GtkSelectionData         *data,
                                                             guint                     info,
-                                                            guint                     time,
+                                                            guint                     drag_time,
                                                             XfceAppfinderWindow      *window);
 static void       _xfce_appfinder_window_execute           (XfceAppfinderWindow      *window);
-static gpointer   _xfce_appfinder_window_monitor_directory (XfceMenu                 *menu,
-                                                            const gchar              *filename,
-                                                            gpointer                  user_data);
-static void       _xfce_appfinder_window_remove_monitor    (XfceMenu                 *menu,
-                                                            gpointer                  monitor_handle);
-static void       _xfce_appfinder_window_something_changed (ThunarVfsMonitor         *monitor,
-                                                            ThunarVfsMonitorHandle   *handle,
-                                                            ThunarVfsMonitorEvent     event,
-                                                            ThunarVfsPath            *handle_path,
-                                                            ThunarVfsPath            *event_path,
-                                                            gpointer                  user_data);
 static void       _xfce_appfinder_window_load_menu_item    (XfceAppfinderWindow      *window,
-                                                            XfceMenuItem             *item,
+                                                            GarconMenuItem             *item,
                                                             const gchar              *category,
                                                             gint                     *counter);
-static GdkPixbuf *_xfce_appfinder_window_create_item_icon  (XfceMenuItem             *item);
+static GdkPixbuf *_xfce_appfinder_window_create_item_icon  (GarconMenuItem             *item);
 static gboolean   _xfce_appfinder_window_visible_func      (GtkTreeModel             *filter,
                                                             GtkTreeIter              *iter,
                                                             gpointer                  user_data);
@@ -133,12 +121,12 @@ static void       _xfce_appfinder_window_set_category      (XfceAppfinderWindow
 
 struct _XfceAppfinderWindowClass
 {
-  GtkWindowClass __parent__;
+  XfceTitledDialogClass __parent__;
 };
 
 struct _XfceAppfinderWindow
 {
-  GtkWindow      __parent__;
+  XfceTitledDialog __parent__;
 
   XfconfChannel *channel;
 
@@ -153,8 +141,8 @@ struct _XfceAppfinderWindow
   GtkListStore  *list_store;
   GtkTreeModel  *filter;
   GtkWidget     *tree_view;
-  
-  XfceMenu      *menu;
+
+  GarconMenu    *menu;
   gchar         *menu_filename;
 
   GThread       *reload_thread;
@@ -162,48 +150,13 @@ struct _XfceAppfinderWindow
 
 
 
-static GObjectClass *xfce_appfinder_window_parent_class = NULL;
-
 static const GtkTargetEntry dnd_target_list[] = {
   { "text/uri-list", 0, 0 }
 };
 
 
 
-XfceMenuMonitorVTable menu_monitor_vtable = {
-  NULL, 
-  _xfce_appfinder_window_monitor_directory,
-  _xfce_appfinder_window_remove_monitor,
-};
-
-
-
-GType
-xfce_appfinder_window_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info = 
-        {
-          sizeof (XfceAppfinderWindowClass),
-          NULL,
-          NULL,
-          (GClassInitFunc) xfce_appfinder_window_class_init,
-          NULL,
-          NULL,
-          sizeof (XfceAppfinderWindow),
-          0,
-          (GInstanceInitFunc) xfce_appfinder_window_init,
-          NULL,
-        };
-
-      type = g_type_register_static (GTK_TYPE_WINDOW, "XfceAppfinderWindow", &info, 0);
-    }
-  
-  return type;
-}
+G_DEFINE_TYPE (XfceAppfinderWindow, xfce_appfinder_window, XFCE_TYPE_TITLED_DIALOG)
 
 
 
@@ -212,19 +165,14 @@ xfce_appfinder_window_class_init (XfceAppfinderWindowClass *klass)
 {
   GObjectClass *gobject_class;
 
-  /* Determine parent type class */
-  xfce_appfinder_window_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
-#if GLIB_CHECK_VERSION (2,14,0)
   gobject_class->constructed = xfce_appfinder_window_constructed;
-#endif
   gobject_class->dispose = xfce_appfinder_window_dispose;
   gobject_class->finalize = xfce_appfinder_window_finalize;
   gobject_class->get_property = xfce_appfinder_window_get_property;
   gobject_class->set_property = xfce_appfinder_window_set_property;
 
-  g_object_class_install_property (gobject_class, 
+  g_object_class_install_property (gobject_class,
                                    PROP_MENU_FILENAME,
                                    g_param_spec_string ("menu-filename",
                                                         "menu-filename",
@@ -233,7 +181,7 @@ xfce_appfinder_window_class_init (XfceAppfinderWindowClass *klass)
                                                         G_PARAM_CONSTRUCT_ONLY |
                                                         G_PARAM_WRITABLE));
 
-  g_object_class_install_property (gobject_class, 
+  g_object_class_install_property (gobject_class,
                                    PROP_CATEGORY,
                                    g_param_spec_string ("category",
                                                         "category",
@@ -250,15 +198,12 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
 {
   GtkTreeViewColumn *column;
   GtkCellRenderer   *renderer;
-  GtkWidget         *vbox1;
   GtkWidget         *vbox2;
   GtkWidget         *vbox3;
   GtkWidget         *hbox1;
   GtkWidget         *hbox2;
-  GtkWidget         *bbox;
-  GtkWidget         *separator;
-  GtkWidget         *heading;
   GtkWidget         *check_button;
+  GtkWidget         *execute_image;
   GtkWidget         *button;
   GtkWidget         *label;
   GtkWidget         *alignment;
@@ -279,6 +224,8 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
 
   gtk_window_set_title (GTK_WINDOW (window), _("Application Finder"));
   gtk_window_set_icon_name (GTK_WINDOW (window), "xfce4-appfinder");
+  xfce_titled_dialog_set_subtitle (XFCE_TITLED_DIALOG (window), _("Find and launch applications installed on your system"));
+  gtk_dialog_set_has_separator (GTK_DIALOG (window), FALSE);
 
   width = xfconf_channel_get_int (window->channel, "/window-width", DEFAULT_WINDOW_WIDTH);
   height = xfconf_channel_get_int (window->channel, "/window-height", DEFAULT_WINDOW_HEIGHT);
@@ -286,24 +233,9 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   gtk_window_set_default_size (GTK_WINDOW (window), width, height);
   gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
 
-  vbox1 = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (window), vbox1);
-  gtk_widget_show (vbox1);
-
-  heading = xfce_heading_new ();
-  xfce_heading_set_title (XFCE_HEADING (heading), _("Application Finder"));
-  xfce_heading_set_subtitle (XFCE_HEADING (heading), _("Find and launch applications installed on your system"));
-  xfce_heading_set_icon_name (XFCE_HEADING (heading), "xfce4-appfinder");
-  gtk_box_pack_start (GTK_BOX (vbox1), heading, FALSE, TRUE, 0);
-  gtk_widget_show (heading);
-
-  separator = gtk_hseparator_new ();
-  gtk_box_pack_start (GTK_BOX (vbox1), separator, FALSE, TRUE, 0);
-  gtk_widget_show (separator);
-
   vbox2 = gtk_vbox_new (FALSE, 6);
   gtk_container_set_border_width (GTK_CONTAINER (vbox2), 6);
-  gtk_container_add (GTK_CONTAINER (vbox1), vbox2);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), vbox2, TRUE, TRUE, 0);
   gtk_widget_show (vbox2);
 
   hbox1 = gtk_hbox_new (FALSE, 12);
@@ -328,8 +260,13 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   gtk_box_pack_start (GTK_BOX (vbox3), alignment, FALSE, TRUE, 0);
   gtk_widget_show (alignment);
 
+#if GTK_CHECK_VERSION (2, 16, 0)
+  window->search_entry = gtk_entry_new ();
+  gtk_entry_set_icon_from_stock (GTK_ENTRY (window->search_entry), GTK_ENTRY_ICON_PRIMARY, GTK_STOCK_FIND);
+#else
   window->search_entry = frap_icon_entry_new ();
   frap_icon_entry_set_stock_id (FRAP_ICON_ENTRY (window->search_entry), GTK_STOCK_FIND);
+#endif
   g_signal_connect (window->search_entry, "changed", G_CALLBACK (_xfce_appfinder_window_entry_changed), window);
   g_signal_connect (window->search_entry, "activate", G_CALLBACK (_xfce_appfinder_window_entry_activated), window);
   g_signal_connect (window->search_entry, "focus-in-event", G_CALLBACK (_xfce_appfinder_window_entry_focused), window);
@@ -357,13 +294,15 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   gtk_container_add (GTK_CONTAINER (hbox1), scrollwin);
   gtk_widget_show (scrollwin);
 
-  window->list_store = gtk_list_store_new (4, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, XFCE_TYPE_MENU_ITEM);
+  window->list_store =
+    gtk_list_store_new (5, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, GARCON_TYPE_MENU_ITEM, G_TYPE_STRING);
 
   window->filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (window->list_store), NULL);
   gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (window->filter), _xfce_appfinder_window_visible_func, window, NULL);
 
   window->tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (window->filter));
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (window->tree_view), FALSE);
+  gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (window->tree_view), TOOLTIP_COLUMN);
   g_signal_connect (window->tree_view, "cursor-changed", G_CALLBACK (_xfce_appfinder_window_cursor_changed), window);
   g_signal_connect_swapped (window->tree_view, "row-activated", G_CALLBACK (_xfce_appfinder_window_execute), window);
   g_signal_connect (window->tree_view, "drag-data-get", G_CALLBACK (_xfce_appfinder_window_drag_data_get), window);
@@ -382,35 +321,36 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   column = gtk_tree_view_column_new_with_attributes (NULL, renderer, "markup", TEXT_COLUMN, NULL);
   gtk_tree_view_append_column (GTK_TREE_VIEW (window->tree_view), GTK_TREE_VIEW_COLUMN (column));
 
-  hbox2 = gtk_hbox_new (FALSE, 12);
-  gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, TRUE, 0);
+  hbox2 = gtk_hbox_new (FALSE, 6);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), hbox2, FALSE, TRUE, 0);
   gtk_widget_show (hbox2);
 
-  check_button = gtk_check_button_new_with_mnemonic (_("C_lose after execute"));
+  check_button = gtk_check_button_new_with_mnemonic (_("C_lose after launch"));
   gtk_box_pack_start (GTK_BOX (hbox2), check_button, FALSE, TRUE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (check_button), 6);
   gtk_widget_show (check_button);
 
   xfconf_g_property_bind (window->channel, "/close-after-execute", G_TYPE_BOOLEAN, G_OBJECT (check_button), "active");
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button), xfconf_channel_get_bool (window->channel, "/close-after-execute", DEFAULT_CLOSE_AFTER_EXECUTE));
 
-  bbox = gtk_hbutton_box_new ();
-  gtk_box_set_spacing (GTK_BOX (bbox), 8);
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
-  gtk_container_add (GTK_CONTAINER (hbox2), bbox);
-  gtk_widget_show (bbox);
-
-  window->execute_button = gtk_button_new_from_stock (GTK_STOCK_EXECUTE);
+  window->execute_button = gtk_button_new_with_mnemonic (_("Launch"));
+  execute_image = gtk_image_new_from_stock (GTK_STOCK_EXECUTE, GTK_ICON_SIZE_BUTTON);
+  gtk_button_set_image (GTK_BUTTON (window->execute_button), execute_image);
+  gtk_dialog_add_action_widget (GTK_DIALOG (window), window->execute_button, GTK_RESPONSE_OK);
   GTK_WIDGET_SET_FLAGS (window->execute_button, GTK_CAN_DEFAULT);
   gtk_button_set_focus_on_click (GTK_BUTTON (window->execute_button), FALSE);
   gtk_widget_set_sensitive (window->execute_button, FALSE);
   g_signal_connect_swapped (window->execute_button, "clicked", G_CALLBACK (_xfce_appfinder_window_execute), window);
-  gtk_container_add (GTK_CONTAINER (bbox), window->execute_button);
   gtk_widget_show (window->execute_button);
 
-  button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
+  button = gtk_dialog_add_button (GTK_DIALOG (window), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
   g_signal_connect_swapped (button, "clicked", G_CALLBACK (_xfce_appfinder_window_closed), window);
-  gtk_container_add (GTK_CONTAINER (bbox), button);
-  gtk_widget_show (button);
+
+  g_object_ref (G_OBJECT (GTK_DIALOG (window)->action_area));
+  gtk_container_remove (GTK_CONTAINER (GTK_DIALOG (window)->vbox),
+                        GTK_DIALOG (window)->action_area);
+  gtk_box_pack_start (GTK_BOX (hbox2), GTK_DIALOG (window)->action_area, TRUE, TRUE, 0);
+  g_object_unref (G_OBJECT (GTK_DIALOG (window)->action_area));
 }
 
 
@@ -419,8 +359,6 @@ static void
 xfce_appfinder_window_constructed (GObject *object)
 {
   XfceAppfinderWindow *window = XFCE_APPFINDER_WINDOW (object);
-  
-  xfce_menu_monitor_set_vtable (&menu_monitor_vtable, window);
 
   xfconf_g_property_bind (window->channel, "/category", G_TYPE_STRING, G_OBJECT (window), "category");
 }
@@ -443,7 +381,7 @@ xfce_appfinder_window_finalize (GObject *object)
   g_free (window->current_category);
   g_free (window->menu_filename);
 
-  if (G_LIKELY (XFCE_IS_MENU (window->menu)))
+  if (G_LIKELY (GARCON_IS_MENU (window->menu)))
     g_object_unref (window->menu);
 
   g_slist_free (window->categories_group);
@@ -505,15 +443,7 @@ xfce_appfinder_window_set_property (GObject      *object,
 GtkWidget *
 xfce_appfinder_window_new (const gchar *filename)
 {
-  GObject *object;
-  
-  object = g_object_new (XFCE_TYPE_APPFINDER_WINDOW, "menu-filename", filename, NULL);
-
-#if !GLIB_CHECK_VERSION (2,14,0)
-  xfce_appfinder_window_constructed (object);
-#endif
-
-  return GTK_WIDGET (object);
+  return g_object_new (XFCE_TYPE_APPFINDER_WINDOW, "menu-filename", filename, NULL);
 }
 
 
@@ -561,14 +491,14 @@ _xfce_appfinder_window_entry_activated (GtkEntry            *entry,
 
 
 
-static void
+static gboolean
 _xfce_appfinder_window_entry_focused (GtkWidget           *entry,
                                       GdkEventFocus       *event,
                                       XfceAppfinderWindow *window)
 {
   GtkTreeSelection *selection;
 
-  g_return_if_fail (XFCE_IS_APPFINDER_WINDOW (window));
+  g_return_val_if_fail (XFCE_IS_APPFINDER_WINDOW (window), FALSE);
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->tree_view));
 
@@ -576,6 +506,8 @@ _xfce_appfinder_window_entry_focused (GtkWidget           *entry,
     gtk_tree_selection_unselect_all (selection);
 
   gtk_widget_set_sensitive (window->execute_button, FALSE);
+
+  return FALSE;
 }
 
 
@@ -678,7 +610,7 @@ _xfce_appfinder_window_view_key_pressed (GtkWidget           *widget,
 
 
 static void
-_xfce_appfinder_window_category_changed (XfceAppfinderWindow *window, 
+_xfce_appfinder_window_category_changed (XfceAppfinderWindow *window,
                                          GtkToggleButton     *button)
 {
   g_return_if_fail (XFCE_IS_APPFINDER_WINDOW (window));
@@ -696,15 +628,15 @@ _xfce_appfinder_window_cursor_changed (GtkTreeView         *tree_view,
   GtkTreeSelection *selection;
   GtkTreeModel     *model;
   GtkTreeIter       iter;
-  XfceMenuItem     *item = NULL;
+  GarconMenuItem     *item = NULL;
 
   g_return_if_fail (XFCE_IS_APPFINDER_WINDOW (window));
 
   selection = gtk_tree_view_get_selection (tree_view);
-  
+
   if (G_LIKELY (selection != NULL && gtk_tree_selection_get_selected (selection, &model, &iter)))
     gtk_tree_model_get (model, &iter, ITEM_COLUMN, &item, -1);
-  
+
   if (G_LIKELY (item != NULL))
     {
       gtk_widget_set_sensitive (window->execute_button, TRUE);
@@ -724,10 +656,10 @@ _xfce_appfinder_window_drag_data_get (GtkWidget           *widget,
                                       GdkDragContext      *drag_context,
                                       GtkSelectionData    *data,
                                       guint                info,
-                                      guint                time,
+                                      guint                drag_time,
                                       XfceAppfinderWindow *window)
 {
-  XfceMenuItem     *item;
+  GarconMenuItem   *item;
   GtkTreeSelection *selection;
   GtkTreeModel     *model;
   GtkTreeIter       iter;
@@ -741,12 +673,11 @@ _xfce_appfinder_window_drag_data_get (GtkWidget           *widget,
     return;
 
   gtk_tree_model_get (model, &iter, ITEM_COLUMN, &item, -1);
-  
+
   if (G_UNLIKELY (item == NULL))
     return;
 
-  uri = g_filename_to_uri (xfce_menu_item_get_filename (item), NULL, NULL);
-
+  uri = garcon_menu_item_get_uri (item);
   if (G_LIKELY (uri != NULL))
     {
       gtk_selection_data_set (data, data->target, 8, (guchar *) uri, strlen (uri));
@@ -759,13 +690,13 @@ _xfce_appfinder_window_drag_data_get (GtkWidget           *widget,
 static void
 _xfce_appfinder_window_execute (XfceAppfinderWindow *window)
 {
-  XfceMenuItem     *item;
+  GarconMenuItem   *item;
   GtkTreeSelection *selection;
   GtkTreeModel     *model;
   GtkTreeIter       iter;
   GdkScreen        *screen;
   GError           *error = NULL;
-  gchar            *command;
+  gchar            *command, *uri;
 
   g_return_if_fail (XFCE_IS_APPFINDER_WINDOW (window));
 
@@ -779,19 +710,19 @@ _xfce_appfinder_window_execute (XfceAppfinderWindow *window)
   if (G_UNLIKELY (item == NULL))
     return;
 
-  command = g_strconcat ("exo-open ", xfce_menu_item_get_filename (item), NULL);
+  uri = garcon_menu_item_get_uri (item);
+  command = g_strconcat ("exo-open ", uri, NULL);
+  g_free (uri);
 
-  screen = xfce_gdk_display_locate_monitor_with_pointer (gdk_display_get_default (), NULL);
+  screen = xfce_gdk_screen_get_active (NULL);
 
-  if (G_UNLIKELY (!xfce_exec_on_screen (screen, command, FALSE, TRUE, &error)))
+  if (G_UNLIKELY (!xfce_spawn_command_line_on_screen (screen, command, FALSE, TRUE, &error)))
     {
-      if (G_LIKELY (error != NULL))
-        {
-          xfce_err (_("Could not execute application %s. Reason: %s"), xfce_menu_element_get_name (XFCE_MENU_ELEMENT (item)), error->message);
-          g_error_free (error);
-        }
-      else
-        xfce_err (_("Could not execute application %s"), xfce_menu_element_get_name (XFCE_MENU_ELEMENT (item)));
+      xfce_dialog_show_error (GTK_WINDOW (window), error,
+           _("Could not execute application %s."), garcon_menu_element_get_name (GARCON_MENU_ELEMENT (item)));
+
+      if (error != NULL)
+        g_error_free (error);
     }
 
   g_free (command);
@@ -822,47 +753,47 @@ _xfce_appfinder_window_closed (XfceAppfinderWindow *window)
 
 static void
 _xfce_appfinder_window_load_menu (XfceAppfinderWindow *window,
-                                  XfceMenu            *menu,
+                                  GarconMenu          *menu,
                                   const gchar         *category,
                                   gint                *counter,
                                   gboolean             is_root,
                                   gboolean             is_category)
 {
-  XfceMenuDirectory *directory;
-  GtkWidget         *button;
-  GSList            *items;
-  GSList            *menus;
-  GSList            *iter;
-  const gchar       *name;
-  gint               current_counter = 0;
+  GarconMenuDirectory *directory;
+  GtkWidget           *button;
+  GList               *items;
+  GList               *menus;
+  GList               *iter;
+  const gchar         *name;
+  gint                 current_counter = 0;
 
-  g_return_if_fail (XFCE_IS_MENU (menu));
+  g_return_if_fail (GARCON_IS_MENU (menu));
   g_return_if_fail (XFCE_IS_APPFINDER_WINDOW (window));
 
-  directory = xfce_menu_get_directory (menu);
+  directory = garcon_menu_get_directory (menu);
 
   if (G_LIKELY (directory != NULL))
-    if (G_UNLIKELY (!xfce_menu_directory_show_in_environment (directory) 
-                    || xfce_menu_directory_get_hidden (directory) 
-                    || xfce_menu_directory_get_no_display (directory)))
+    if (G_UNLIKELY (!garcon_menu_directory_get_show_in_environment (directory)
+                    || garcon_menu_directory_get_hidden (directory)
+                    || garcon_menu_directory_get_no_display (directory)))
       {
         return;
       }
 
   /* Determine menu name */
-  name = xfce_menu_element_get_name (XFCE_MENU_ELEMENT (menu));
+  name = garcon_menu_element_get_name (GARCON_MENU_ELEMENT (menu));
 
   /* Load menu items */
-  items = xfce_menu_get_items (menu);
-  for (iter = items; iter != NULL; iter = g_slist_next (iter))
-    _xfce_appfinder_window_load_menu_item (window, XFCE_MENU_ITEM (iter->data), is_category ? name : category, &current_counter);
-  g_slist_free (items);
-  
+  items = garcon_menu_get_items (menu);
+  for (iter = items; iter != NULL; iter = g_list_next (iter))
+    _xfce_appfinder_window_load_menu_item (window, GARCON_MENU_ITEM (iter->data), is_category ? name : category, &current_counter);
+  g_list_free (items);
+
   /* Load sub-menus */
-  menus = xfce_menu_get_menus (menu);
-  for (iter = menus; iter != NULL; iter = g_slist_next (iter))
-    _xfce_appfinder_window_load_menu (window, XFCE_MENU (iter->data), is_category ? name : category, &current_counter, FALSE, is_root ? TRUE : FALSE);
-  g_slist_free (menus);
+  menus = garcon_menu_get_menus (menu);
+  for (iter = menus; iter != NULL; iter = g_list_next (iter))
+    _xfce_appfinder_window_load_menu (window, GARCON_MENU (iter->data), is_category ? name : category, &current_counter, FALSE, is_root ? TRUE : FALSE);
+  g_list_free (menus);
 
   /* Create category widget */
   if (G_LIKELY (current_counter > 0 && is_category))
@@ -881,7 +812,7 @@ _xfce_appfinder_window_load_menu (XfceAppfinderWindow *window,
             gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 
           window->categories_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
-    
+
           g_signal_connect_swapped (button, "toggled", G_CALLBACK (_xfce_appfinder_window_category_changed), window);
         }
     }
@@ -891,42 +822,41 @@ _xfce_appfinder_window_load_menu (XfceAppfinderWindow *window,
 
 
 
-static gpointer 
+static gpointer
 _xfce_appfinder_window_reload_menu (XfceAppfinderWindow *window)
 {
   GtkWidget *button;
   GError    *error = NULL;
-  GSList    *menus;
   gint       counter = 0;
 
-  g_return_if_fail (XFCE_IS_APPFINDER_WINDOW (window));
+  g_return_val_if_fail (XFCE_IS_APPFINDER_WINDOW (window), NULL);
 
   DBG ("window->menu_filename = %s", window->menu_filename);
 
   if (G_UNLIKELY (window->menu_filename != NULL))
-    window->menu = xfce_menu_new (window->menu_filename, &error);
+    window->menu = garcon_menu_new_for_path (window->menu_filename);
   else
-    window->menu = xfce_menu_get_root (&error);
+    window->menu = garcon_menu_new_applications ();
 
   if (G_UNLIKELY (window->menu == NULL))
     {
-      if (G_LIKELY (error != NULL))
-        {
-          if (G_UNLIKELY (window->menu_filename != NULL))
-            xfce_err (_("Could not load menu from %s. Reason: %s"), window->menu_filename, error->message);
-          else
-            xfce_err (_("Could not load system menu"));
-
-          g_error_free (error);
-        }
+      if (G_UNLIKELY (window->menu_filename != NULL))
+        xfce_dialog_show_error (GTK_WINDOW (window), NULL,
+          _("Could not load menu from %s"), window->menu_filename);
       else
-        {
-          if (G_UNLIKELY (window->menu_filename != NULL))
-            xfce_err (_("Could not load menu from %s"), window->menu_filename);
-          else
-            xfce_err (_("Could not load system menu"));
-        }
+        xfce_dialog_show_error (GTK_WINDOW (window), NULL,
+          _("Could not load system menu"));
 
+      if (error != NULL)
+        g_error_free (error);
+
+      return NULL;
+    }
+
+  if (!garcon_menu_load (window->menu, NULL, &error))
+    {
+      g_message ("failed to load the menu: %s", error->message);
+      g_error_free (error);
       return NULL;
     }
 
@@ -956,131 +886,87 @@ _xfce_appfinder_window_reload_menu (XfceAppfinderWindow *window)
 
 
 
-static gpointer 
-_xfce_appfinder_window_monitor_directory (XfceMenu    *menu,
-                                          const gchar *filename,
-                                          gpointer     user_data)
-{
-  XfceAppfinderWindow    *window = user_data;
-  ThunarVfsMonitorHandle *monitor_handle = NULL;
-  ThunarVfsMonitor       *monitor;
-  ThunarVfsPath          *path;
-
-  g_return_val_if_fail (XFCE_IS_MENU (menu), NULL);
-  g_return_val_if_fail (XFCE_IS_APPFINDER_WINDOW (window), NULL);
-  g_return_val_if_fail (filename != NULL, NULL);
-
-  if (G_UNLIKELY (!g_file_test (filename, G_FILE_TEST_IS_DIR)))
-    return NULL;
-
-  path = thunar_vfs_path_new (filename, NULL);
-
-  if (G_LIKELY (path != NULL))
-    {
-      monitor = thunar_vfs_monitor_get_default ();
-      monitor_handle = thunar_vfs_monitor_add_directory (monitor, path, _xfce_appfinder_window_something_changed, window);
-      g_object_unref (monitor);
-
-      thunar_vfs_path_unref (path);
-    }
-
-  DBG ("monitor_handle = %p", monitor_handle);
-
-  return monitor_handle;
-}
-
-
-
-static void
-_xfce_appfinder_window_remove_monitor (XfceMenu *menu,
-                                       gpointer  monitor_handle)
-{
-  ThunarVfsMonitor *monitor;
-
-  g_return_if_fail (XFCE_IS_MENU (menu));
-  g_return_if_fail (monitor_handle != NULL);
-
-  DBG ("monitor_handle = %p", monitor_handle);
-
-  monitor = thunar_vfs_monitor_get_default ();
-  thunar_vfs_monitor_remove (monitor, (ThunarVfsMonitorHandle *) monitor_handle);
-  g_object_unref (monitor);
-}
-
-
-
-static void
-_xfce_appfinder_window_something_changed (ThunarVfsMonitor       *monitor,
-                                          ThunarVfsMonitorHandle *handle,
-                                          ThunarVfsMonitorEvent   event,
-                                          ThunarVfsPath          *handle_path,
-                                          ThunarVfsPath          *event_path,
-                                          gpointer                user_data)
-{
-  XfceAppfinderWindow *window = user_data;
-  XfceMenuItemCache   *cache;
-
-  g_return_if_fail (XFCE_IS_APPFINDER_WINDOW (window));
-
-  cache = xfce_menu_item_cache_get_default ();
-  xfce_menu_item_cache_invalidate (cache);
-  g_object_unref (cache);
-
-  if (XFCE_IS_MENU (window->menu))
-    g_object_unref (window->menu);
-
-  gtk_list_store_clear (window->list_store);
-
-  _xfce_appfinder_window_reload_menu (window);
-}
-
-
-
 static void
 _xfce_appfinder_window_load_menu_item (XfceAppfinderWindow *window,
-                                       XfceMenuItem        *item,
+                                       GarconMenuItem        *item,
                                        const gchar         *category,
                                        gint                *counter)
 {
-  GtkTreeIter  iter;
-  GdkPixbuf   *pixbuf;
-  const gchar *name = NULL;
-  const gchar *comment;
-  gchar       *text;
+  GtkTreeIter   iter;
+  GdkPixbuf    *pixbuf;
+  GString      *tooltip_str;
+  GList        *categories;
+  GList        *lp;
+  const gchar  *name = NULL;
+  const gchar  *comment;
+  const gchar  *command;
+  gchar       **categories_array;
+  gchar        *categories_string;
+  gchar        *text;
+  gchar        *tooltip = NULL;
+  guint         n;
 
   g_return_if_fail (XFCE_IS_APPFINDER_WINDOW (window));
 
-  if (G_UNLIKELY (!xfce_menu_item_show_in_environment (item) || xfce_menu_item_get_no_display (item)))
+  if (G_UNLIKELY (!garcon_menu_item_get_show_in_environment (item) || garcon_menu_item_get_no_display (item)))
     return;
 
-  if (G_UNLIKELY (xfce_menu_item_only_show_in_environment (item) || xfce_menu_item_has_category (item, "X-XFCE")))
+  if (G_UNLIKELY (garcon_menu_item_only_show_in_environment (item) || garcon_menu_item_has_category (item, "X-XFCE")))
     {
-      name = xfce_menu_item_get_generic_name (item);
+      name = garcon_menu_item_get_generic_name (item);
 
       if (G_UNLIKELY (name == NULL))
-        name = xfce_menu_element_get_name (XFCE_MENU_ELEMENT (item));
-    }   
+        name = garcon_menu_element_get_name (GARCON_MENU_ELEMENT (item));
+    }
   else
-    name = xfce_menu_element_get_name (XFCE_MENU_ELEMENT (item));
+    name = garcon_menu_element_get_name (GARCON_MENU_ELEMENT (item));
 
-  comment = xfce_menu_item_get_comment (item);
+  comment = garcon_menu_item_get_comment (item);
   pixbuf = _xfce_appfinder_window_create_item_icon (item);
-
-  DBG ("name = %s", name);
+  categories = garcon_menu_item_get_categories (item);
+  command = garcon_menu_item_get_command (item);
 
   if (G_LIKELY (comment != NULL))
     text = g_strdup_printf ("<b>%s</b>\n%s", name, comment);
   else
     text = g_strdup_printf ("<b>%s</b>", name);
 
+  tooltip_str = g_string_new (NULL);
+
+  if (G_LIKELY (categories != NULL))
+    {
+      categories_array = g_new0 (gchar *, g_list_length (categories) + 1);
+
+      for (lp = categories, n = 0; lp != NULL; lp = lp->next, ++n)
+        categories_array[n] = lp->data;
+
+      categories_string = g_strjoinv (", ", categories_array);
+      g_string_append_printf (tooltip_str, _("<b>Categories:</b> %s"), categories_string);
+      g_free (categories_string);
+
+      g_free (categories_array);
+    }
+
+  if (command != NULL && *command != '\0')
+    {
+      if (categories != NULL)
+        g_string_append_c (tooltip_str, '\n');
+
+      g_string_append_printf (tooltip_str, _("<b>Command:</b> %s"), command);
+    }
+
+  tooltip = g_string_free (tooltip_str, FALSE);
+
   gtk_list_store_append (window->list_store, &iter);
-  gtk_list_store_set (window->list_store, &iter, 
-                      ICON_COLUMN, pixbuf, 
+  gtk_list_store_set (window->list_store, &iter,
+                      ICON_COLUMN, pixbuf,
                       TEXT_COLUMN, text,
                       CATEGORY_COLUMN, category,
-                      ITEM_COLUMN, item, -1);
+                      ITEM_COLUMN, item,
+                      TOOLTIP_COLUMN, tooltip,-1);
 
   g_free (text);
+  g_free (tooltip);
 
   if (GDK_IS_PIXBUF (pixbuf))
     g_object_unref (pixbuf);
@@ -1091,13 +977,13 @@ _xfce_appfinder_window_load_menu_item (XfceAppfinderWindow *window,
 
 
 static GdkPixbuf *
-_xfce_appfinder_window_create_item_icon (XfceMenuItem *item)
+_xfce_appfinder_window_create_item_icon (GarconMenuItem *item)
 {
   GdkPixbuf    *icon = NULL;
   GtkIconTheme *icon_theme;
   const gchar  *icon_name;
   const gchar  *item_name;
-  gchar        *basename;
+  gchar        *base_name;
   gchar        *extension;
   gchar        *new_item_name;
   gchar         new_icon_name[1024];
@@ -1105,8 +991,8 @@ _xfce_appfinder_window_create_item_icon (XfceMenuItem *item)
   /* Get current icon theme */
   icon_theme = gtk_icon_theme_get_default ();
 
-  item_name = xfce_menu_element_get_name (XFCE_MENU_ELEMENT (item));
-  icon_name = xfce_menu_element_get_icon_name (XFCE_MENU_ELEMENT (item));
+  item_name = garcon_menu_element_get_name (GARCON_MENU_ELEMENT (item));
+  icon_name = garcon_menu_element_get_icon_name (GARCON_MENU_ELEMENT (item));
 
   if (icon_name == NULL)
     return NULL;
@@ -1123,23 +1009,23 @@ _xfce_appfinder_window_create_item_icon (XfceMenuItem *item)
       if (icon == NULL)
         {
           /* Get basename (just to be sure) */
-          basename = g_path_get_basename (icon_name);
+          base_name = g_path_get_basename (icon_name);
 
           /* Determine position of the extension */
-          extension = g_utf8_strrchr (basename, -1, '.');
+          extension = g_utf8_strrchr (base_name, -1, '.');
 
           /* Make sure we found an extension */
           if (extension != NULL)
             {
               /* Remove extension */
-              g_utf8_strncpy (new_icon_name, basename, g_utf8_strlen (basename, -1) - g_utf8_strlen (extension, -1));
+              g_utf8_strncpy (new_icon_name, base_name, g_utf8_strlen (base_name, -1) - g_utf8_strlen (extension, -1));
 
               /* Try to load the pixbuf using the new icon name */
               icon = gtk_icon_theme_load_icon (icon_theme, new_icon_name, ICON_SIZE, GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
             }
 
           /* Free basename */
-          g_free (basename);
+          g_free (base_name);
 
           /* As a last fallback, we try to load the icon by lowercase item name */
           if (icon == NULL && item_name != NULL)
@@ -1174,15 +1060,20 @@ _xfce_appfinder_window_visible_func (GtkTreeModel *filter,
                                      gpointer      user_data)
 {
   XfceAppfinderWindow *window = user_data;
+  GarconMenuItem      *item;
   gchar               *category;
   gchar               *text;
   gchar               *search_text;
   gchar               *normalized;
+  gchar               *command;
   gboolean             visible = FALSE;
 
   g_return_val_if_fail (XFCE_IS_APPFINDER_WINDOW (window), FALSE);
 
-  gtk_tree_model_get (filter, iter, CATEGORY_COLUMN, &category, TEXT_COLUMN, &text, -1);
+  gtk_tree_model_get (filter, iter,
+                      CATEGORY_COLUMN, &category,
+                      TEXT_COLUMN, &text,
+                      ITEM_COLUMN, &item, -1);
 
   if (G_UNLIKELY (text == NULL))
     return FALSE;
@@ -1196,9 +1087,21 @@ _xfce_appfinder_window_visible_func (GtkTreeModel *filter,
   text = g_utf8_casefold (normalized, -1);
   g_free (normalized);
 
-  if (G_UNLIKELY (g_strstr_len (text, -1, search_text) != NULL))
+  if (garcon_menu_item_get_command (item) != NULL)
     {
-      if (window->current_category == NULL || 
+      normalized = g_utf8_normalize (garcon_menu_item_get_command (item), -1, G_NORMALIZE_ALL);
+      command = g_utf8_casefold (normalized, -1);
+      g_free (normalized);
+    }
+  else
+    {
+      command = g_strdup ("");
+    }
+
+  if (g_strstr_len (text, -1, search_text) != NULL ||
+      g_strstr_len (command, -1, search_text) != NULL)
+    {
+      if (window->current_category == NULL ||
           g_utf8_strlen (window->current_category, -1) == 0 ||
           g_utf8_collate (window->current_category, _("All")) == 0)
         {
@@ -1206,12 +1109,13 @@ _xfce_appfinder_window_visible_func (GtkTreeModel *filter,
         }
       else
         {
-          if (G_UNLIKELY (category != NULL && g_utf8_collate (category, window->current_category) == 0))
+          if (category != NULL && g_utf8_collate (category, window->current_category) == 0)
             visible = TRUE;
         }
     }
 
   g_free (search_text);
+  g_free (command);
   g_free (text);
   g_free (category);
 
